#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File              : io_utils.py
# Author            : Jiahao Yao
# Email             : jiahaoyao.math@gmail.com
# Date              : 03.16.2021
# Last Modified Date: 03.16.2020
# Last Modified By  : Jiahao Yao
#
# This file is part of the sysflow codebase
# Distributed under MIT license
#
# python basic code template utilities
from sysflow.utils.common_utils.io_utils import cprint
import re
import argparse

def str2bool(v):
    if isinstance(v, bool):
       return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')


def pack_params(*params, header=True):
    # https://github.com/Microsoft/vscode/issues/12146
    lines = []
    for param in params: 
        assert isinstance(param, str) 
        # exec('self.{} = {}'.format(param, param))
        lines.append('self.{} = {}'.format(param, param))
    if header: 
        lines = ['#region save the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))

def load_params(*params, header=True):
    lines = []
    for param in params: 
        assert isinstance(param, str) 
        # exec('{} = self.{}'.format(param, param))
        lines.append('{} = self.{}'.format(param, param))
    if header: 
        lines = ['#region unpack the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))

def pack_dict(d, *params, header=True):
    lines = []
    for param in params: 
        assert isinstance(param, str) 
        # exec("{}['{}'] = {}".format(d, param, param))
        lines.append("{}['{}'] = {}".format(d, param, param))
    if header: 
        lines = ['#region save the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))
        
def load_dict(d, *params, header=True):
    lines = []
    for param in params: 
        assert isinstance(param, str) 
        # exec("{} = {}['{}']".format(param, d, param), globals())
        lines.append("{} = {}['{}']".format(param, d, param))
    if header: 
        lines = ['#region unpack the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))

def load_hp(*params, header=True):
    # load the parameter for hpman      
    lines = []
    for param in params: 
        assert isinstance(param, str) 
        # exec("{} = {}['{}']".format(param, d, param), globals())
        lines.append("{} = _.get_value('{}')".format(param, param))
    if header: 
        lines = ['#region unpack the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))

def arg2hp(s, header=True):
    # argparse to hpman
    s = s.replace("\"", "'")
    lines = []
    for line in s.split('\n'): 
        line = line.strip()
        if len(line) and line[0]!='#':
            var = re.findall(r"'--(.*?)'", line)[0] 
            val = re.findall(r"default=(.*)\)", line)[0] 
            lines.append("{} = _('{}', {})".format(var, var, val))
    if header: 
        lines = ['#region hpman parameters'] + lines + ['#endregion']
    cprint('\n'.join(lines))

def hp2arg(s, header=True):
    # hpman to argparse
    s = s.replace("\"", "'")
    lines = []
    extra_head = None
    for line in s.split('\n'): 
        line = line.strip()
        if len(line) and line[0]!='#':
            var, val = re.findall(r"\('(.*?)',\ (.*?)\)", line)[0]
            pyval = eval(val, {"__builtins__":None})
            # https://stackoverflow.com/questions/9072847/interpreting-strings-as-other-data-types-in-python
            val_type = type(pyval).__name__
            if val_type == 'bool': 
                val_type = 'str2bool'
                extra_head = 'from sysflow.utils.common_utils.code_utils import str2bool'
            lines.append("parser.add_argument('--{}', type={}, default={})".format(var, val_type, val))
    if extra_head: 
        lines = [extra_head] + lines

    if header: 
        lines = ['#region argparse parameters'] + lines + ['#endregion']
    cprint('\n'.join(lines))

def load_hp2hp(s, header=True):
    # load the parameter for hpman      
    lines = []
    s = s.replace("\"", "'")
    for line in s.split('\n'): 
        line = line.strip()
        if len(line) and line[0]!='#':
            var = re.findall(r"\('(.*?)',", line)[0] 
            lines.append("{} = _.get_value('{}')".format(var, var))
    if header: 
        lines = ['#region unpack the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))


# colab function 
def hp2colab(s):
    # hpman to colab notebook
    TYPE_COLAB = {
        'int': 'integer', 
        'float': 'number', 
        'bool': 'boolean', 
        'str': 'string'
    }
    s = s.replace("\"", "'")
    lines = []
    var_list = []
    for line in s.split('\n'): 
        line = line.strip()
        if len(line) and line[0]!='#':
            var, val = re.findall(r"\('(.*?)',\ (.*?)\)", line)[0]
            var_list.append(var)
            pyval = eval(val, {"__builtins__":None})
            # https://stackoverflow.com/questions/9072847/interpreting-strings-as-other-data-types-in-python
            val_type = type(pyval).__name__
            val_type = TYPE_COLAB[val_type]
            lines.append("{} = {} #@param {{type:'{}'}}".format(var, val, val_type))

    lines.append('\nfrom argparse import Namespace')

    namespace_line = ['{}={}'.format(var, var) for var in var_list]
    lines.append('args = Namespace({})'.format(', '.join(namespace_line)))
    
    cprint('\n'.join(lines))


def arg2colab(s):
    # hpman to colab notebook
    TYPE_COLAB = {
        'int': 'integer', 
        'float': 'number', 
        'str2bool': 'boolean', 
        'str': 'string'
    }
    s = s.replace("\"", "'")
    lines = []
    var_list = []
    for line in s.split('\n'): 
        line = line.strip()
        if len(line) and line[0]!='#':
            var, val_type, val = re.findall(r"'--(.*?)', *type=(.*?), *default=(.*?)[,\)]", line)[0]
            var_list.append(var)
            val_type = TYPE_COLAB[val_type]
            if 'help' in line: 
                comment = re.findall(r"help='(.*?)'\)", line)[0]
                lines.append("# {} \n{} = {} #@param {{type:'{}'}}".format(comment, var, val, val_type))

            else: 
                lines.append("{} = {} #@param {{type:'{}'}}".format(var, val, val_type))
        else: 
            if len(line) and line[0] == '#': 
                lines.append('#@markdown' + line[1:])
            else: 
                lines.append(line)
    lines.append('\nfrom argparse import Namespace')
    
    namespace_line = ['{}={}'.format(var, var) for var in var_list]
    lines.append('args = Namespace({})'.format(', '.join(namespace_line)))
    
    cprint('\n'.join(lines))


def load_hp2arg(s, header=True):
    # load the parameter using hpman to using argparse
    lines = []
    s = s.replace("\"", "'")
    for line in s.split('\n'): 
        line = line.strip()
        if len(line) and line[0]!='#':
            var = re.findall(r"\('(.*?)'\)", line)[0] 
            lines.append("{} = args.{}".format(var, var))
    if header: 
        lines = ['#region unpack the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))

def load_hp2self(s, header=True):
    # load the parameter using hpman to using argparse
    lines = []
    s = s.replace("\"", "'")
    for line in s.split('\n'): 
        line = line.strip()
        if len(line) and line[0]!='#':
            var = re.findall(r"\('(.*?)'\)", line)[0] 
            lines.append("self.{} = {}".format(var, var))
    if header: 
        lines = ['#region unpack the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))

def load_hp2pack_self(s, header=True):
    # load the parameter using hpman to using argparse
    lines = []
    s = s.replace("\"", "'")
    for line in s.split('\n'): 
        line = line.strip()
        if len(line) and line[0]!='#':
            var = re.findall(r"\('(.*?)'\)", line)[0] 
            lines.append("{} = self.{}".format(var, var))
    if header: 
        lines = ['#region unpack the params'] + lines + ['#endregion']
    cprint('\n'.join(lines))


def get_uniq(s): 
    # remove the repeating arguments (because we might paste the code everywhere)
    s_list = s.split('\n')
    unique_s_list = []
    for ss in s_list: 
        if ( len(ss.strip()) > 0 ) and (ss not in unique_s_list) and (ss.strip()[0]!= '#'):
            unique_s_list.append(ss)
    cprint('\n'.join(unique_s_list))

if __name__ == '__main__':
    a = 1
    b = 2 
    mydict = {}
    pack_dict('mydict', 'b')

    mydict2 = {}
    pack_dict('mydict2', 'a', 'b')


    mydict3 = {'ab': 2, 'cd': 5}
    load_dict('mydict3', 'ab', 'cd')

    class mycls: 
        u = 1
        bb = 2
        pack_params('u', 'bb')

    myclass = mycls()

    load_params('u', 'bb')

    load_params('device', 'D', 'G', 'G_optimizer', 'D_optimizer', header=False)

    load_hp('device', 'D')

    s = """
    parser.add_argument("--niters", type=int, default=10000)
    parser.add_argument("--batch_size", type=int, default=2000)
    parser.add_argument("--lr", type=float, default=3e-4)
    parser.add_argument("--weight_decay", type=float, default=0)
    parser.add_argument("--critic_weight_decay", type=float, default=0)
    parser.add_argument("--save", type=str, default="data/test_steinNS")
    parser.add_argument("--viz_freq", type=int, default=1000)
    parser.add_argument("--d_iters", type=int, default=5)
    parser.add_argument("--g_iters", type=int, default=1)
    parser.add_argument("--l2", type=float, default=10.0)
    """

    arg2hp(s)

    ss = """
    niters = _.('niters', 10000)
    batch_size = _.('batch_size', 2000)
    lr = _.('lr', 3e-4)
    weight_decay = _.('weight_decay', 0)
    critic_weight_decay = _.('critic_weight_decay', 0)
    save = _.('save', "data/test_steinNS")
    viz_freq = _.('viz_freq', 1000)
    d_iters = _.('d_iters', 5)
    g_iters = _.('g_iters', 1)
    l2 = _.('l2', 10.0)
    """

    hp2arg(ss)

    load_hp2hp(ss)

    hp2colab(ss)


    s = """
            niters = args.niters
            batch_size = args.batch_size
            lr = args.lr
            weight_decay = args.weight_decay
            critic_weight_decay = args.critic_weight_decay
            save = args.save
            viz_freq = args.viz_freq
            d_iters = args.d_iters
            g_iters = args.g_iters
            l2 = args.l2
            r = args.r
            sd = args.sd
            n_comp = args.n_comp
            z_dim = args.z_dim
            lr_D = args.lr_D
            out_dim = args.out_dim
            hid_dim = args.hid_dim
            model = args.model
            dim = args.dim
            G_path = args.G_path
            D_path = args.D_path
            use_spectrum = args.use_spectrum
            #endregion

            #region unpack the params
            niters = args.niters
            batch_size = args.batch_size
            lr = args.lr
            weight_decay = args.weight_decay
            critic_weight_decay = args.critic_weight_decay
            save = args.save
            viz_freq = args.viz_freq
            d_iters = args.d_iters
            g_iters = args.g_iters
            l2 = args.l2
            r = args.r
            sd = args.sd
            n_comp = args.n_comp
            z_dim = args.z_dim
            lr_D = args.lr_D
            out_dim = args.out_dim
            hid_dim = args.hid_dim
            model = args.model
            dim = args.dim
            clip_D = args.clip_D
            clip_value = args.clip_value
            loss_A = args.loss_A
            mmd_ratio_in = args.mmd_ratio_in
            mmd_two_sample = args.mmd_two_sample
            mmd_beta = args.mmd_beta
            G_loss = args.G_loss
    """
    get_uniq(s)


    s = """
        # problem setup
        parser.add_argument('--model', type=str, default='mb', choices=['mb', 'dw', 'gaussian'], help='which trajectories we are using for the datasets')
        parser.add_argument('--dim', type=int, default=2, help='the dimension of the dataset')
        parser.add_argument('--gen_thres_eng', type=float, default=100.0, help='the energy ratio for generating the initial data')

        # GAN
        parser.add_argument('--clip_D', type=str2bool, default=False, help='whether to clip the parameter for the discriminator')
        parser.add_argument('--clip_value', type=float, default=.1, help='the bounds for the clipping parameter of the discriminator')
        parser.add_argument('--use_spectrum', type=str2bool, default=False, help='whether to use spectrum normalization in the discriminator to force the lipschitz constant')
        parser.add_argument('--disc_zero', type=str2bool, default=True, help='whether to penalize the l2 penalty towards zero; otherwise penalize the norm to one')
        parser.add_argument('--l2', type=float, default=10.0, help='the coefficient of l2 penality in the loss of generator')
        parser.add_argument('--d_iters', type=int, default=5, help='the number of iterations for discriminator every round')
        parser.add_argument('--g_iters', type=int, default=1, help='the number of iterations for generators every round')

        # MMD
        parser.add_argument('--mmd_ratio_in', type=str2bool, default=False, help='whethe to put the partition function inside when computing the mmd')
        parser.add_argument('--mmd_two_sample', type=str2bool, default=False, help='whether to use two independent groups of samples to estimate the mmd')
        parser.add_argument('--mmd_beta', type=float, default=1.0, help='the parameter of gaussian kernel: std of the gaussian kernel is sigma = 1 / sqrt( 2 * beta )')

        # network
        parser.add_argument('--hid_dim', type=int, default=100, help='the dimension for the hidden layer in the neural network')

        # training 
        parser.add_argument('--loss_A', type=str2bool, default=True, help='whether to use loss type A: i.e. the 1st formula for the loss; otherwise, use the 2nd formula for the loss')
        parser.add_argument('--G_loss', type=str, default='GAN', choices=['GAN', 'MMD', 'GAN+MMD'], help='the type of loss for the generator: 1. GAN --- use the GAN formula for training; 2. MMD -- use the MMD for the generator (i.e. not training the discriminator); 3. GAN+MMD --- combine the both loss functions')
        parser.add_argument('--niters', type=int, default=10000, help='the number of iterations for training the neural networks')
        parser.add_argument('--batch_size', type=int, default=2000, help='the batch size for training the network')
        parser.add_argument('--lr', type=float, default=3e-4, help='the learning for the generator')
        parser.add_argument('--lr_D', type=float, default=1e-4, help='the learning rate of discriminator')

        # utils 
        parser.add_argument('--viz_freq', type=int, default=1000, help='the frequency of generating the visualization data')
        parser.add_argument('--log_freq', type=int, default=1000, help='the frequency of logging during the training')
        parser.add_argument('--G_path', type=str, default='', help='if given, load the generator from the G-path')
        parser.add_argument('--D_path', type=str, default='', help='if given, load the discriminator from the D-path')
    """

    arg2colab(s)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File              : io_utils.py
# Author            : Jiahao Yao
# Email             : jiahaoyao.math@gmail.com
# Date              : 03.16.2021
# Last Modified Date: 03.16.2020
# Last Modified By  : Jiahao Yao
#
# This file is part of the sysflow codebase
# Distributed under MIT license
#
# python basic io utilities
import pyperclip

def cprint(a):
    print(a)
    # safe 
    try: 
        pyperclip.copy(a)
    except: 
        pass


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File              : basic_utils.py
# Author            : Jiahao Yao
# Email             : jiahaoyao.math@gmail.com
# Date              : 09.08.2019
# Last Modified Date: 10.21.2020
# Last Modified By  : Jiahao Yao
#
# This file is part of the VCML codebase
# Distributed under MIT license
#
# python basic utilities

import sys
import numpy as np
import itertools
import operator

def dict_gather(dicts):
    summon = {}
    for d in dicts:
        for key, value in d.items():
            if key not in summon:
                summon[key] = []
            summon[key].append(value)
    output = {
        key: np.array(values)
        for key, values in summon.items()
    }
    return output

def merge_dicts(d1, d2):
    # merge two dicts without duplication
    # https://stackoverflow.com/questions/31323388/merging-two-dicts-in-python-with-no-duplication-permitted
    return {} if any(d1[k] != d2[k] for k in d1.keys() & d2) else dict(d1, **d2)

def contains(x, elements):
    for e in elements:
        if e in x:
            return True
    return False


def only_contains(x, elements):
    for y in x:
        if y not in elements:
            return False
    return True


def belongs_to(stats, query):
    for cat, items in stats.items():
        if query in items:
            return cat
    return None


def intersection(*arg, as_set=False):
    """
    Taking the intersection of multiple iterables.
    """
    output = arg[0]

    if as_set:
        output = set(output)
    else:
        # as list
        output = list(output)

    for y in arg[1:]:
        if as_set:
            output = output.intersection(set(y))
        else:
            set_y = set(y)
            output = [i for i in output if i in set_y]

    return output


def union(*arg, as_set=False):
    """
    Taking the union of multiple iterables
    If the first input is set, or `as_set` is True, the output will be cast
    to a set variable. Otherwise, the output will be a list instance
    """

    output = arg[0]

    if as_set:
        output = set(output)
    else:
        # as list
        output = list(output)

    for y in arg[1:]:
        if as_set:
            output = output.union(set(y))
        else:
            set_output = set(output)
            output = output + [i for i in y if i not in set_output]

    return output


def sum_list(*arg):
    output = arg[0]
    for y in arg[1:]:
        output = output + y
    return output


def difference(x, y):
    # only set or list supported
    if isinstance(x, set):
        return x.difference(set(y))
    else:
        set_y = set(y)
        return [i for i in x if i not in set_y]


def get_size(obj, seen=None):
    """Recursively finds size of objects"""
    size = sys.getsizeof(obj)
    if seen is None:
        seen = set()
    obj_id = id(obj)
    if obj_id in seen:
        return 0
    seen.add(obj_id)
    if isinstance(obj, dict):
        size += sum([get_size(v, seen) for v in obj.values()])
        size += sum([get_size(k, seen) for k in obj.keys()])
    elif hasattr(obj, '__dict__'):
        size += get_size(obj.__dict__, seen)
    elif hasattr(obj, '__iter__') and\
            not isinstance(obj, (str, bytes, bytearray)):
        size += sum([get_size(i, seen) for i in obj])
    return size


class matrix_dict:
    """
    A matrix-like dict class
    Each query takes two keys
    """
    def __init__(self, keys_x, keys_y, values):
        self.keys_x = keys_x
        self.keys_y = keys_y
        self.values = values
        self.build_dict()
        self.assert_values()

    def build_dict(self):
        self.dict_x = dict(zip(self.keys_x, range(self.dim_x)))
        self.dict_y = dict(zip(self.keys_y, range(self.dim_y)))

    def assert_values(self):
        values = self.values
        assert isinstance(values, list) and len(values) == self.dim_x
        for y_values in values:
            assert isinstance(y_values, list)
            assert len(y_values) == self.dim_y

    @property
    def dim_x(self):
        return len(self.keys_x)

    @property
    def dim_y(self):
        return len(self.keys_y)

    def __getitem__(self, query):
        query_x, query_y = query
        output = self.values[
            self.dict_x[query_x]
        ][
            self.dict_y[query_y]
        ]
        return output


#python utils
def sortB(a, b, reverse=False):
    # sort A with B
    ab = sorted(zip(a, b), key=operator.itemgetter(1), reverse=reverse)
    a2, b2 = zip(*ab)
    if isinstance(a, str):
        return str(a2), str(b2)
    elif isinstance(a, list):
        return list(a2), list(b2)
    elif isinstance(a, np.ndarray):
        return np.array(a2), np.array(b2)
    else:
        return a2, b2

def unilist(a):
    return list(set(a))

def unilistB(a, b=None):
    # https://stackoverflow.com/questions/12897374/get-unique-values-from-a-list-in-python
    # filter list a with unique b
    used = set()
    if b == None:
        b = a
    unique = [aa for aa, bb in zip(a,b) if bb not in used and (used.add(bb) or True)]
    return unique

def chunklist(seq, num):
    # https://stackoverflow.com/questions/2130016/splitting-a-list-into-n-parts-of-approximately-equal-length
    k, m = divmod(len(seq), num)
    return (seq[i * k + min(i, m):(i + 1) * k + min(i + 1, m)] for i in range(num))

# flatten the list of lists
# https://stackoverflow.com/questions/952914/how-to-make-a-flat-list-out-of-list-of-lists?page=1&tab=active#tab-top
flattenlist = lambda l: [item for sublist in l for item in sublist]

def concatlist(l_list):
    # https://stackoverflow.com/questions/716477/join-list-of-lists-in-python
    return list(itertools.chain.from_iterable(l_list))

def translist(l_list):
    # https://stackoverflow.com/questions/23940937/reshape-list-of-lists-based-on-position-of-the-element
    return [list(x) for x in zip(*l_list)]

# transpose the list of tuples
# https://stackoverflow.com/questions/43437240/pythonic-way-to-flip-a-list-tuple
transpose_list = lambda t: list(zip(*t))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File              : file_utils.py
# Author            : Chi Han, Jiayuan Mao, Jiahao Yao
# Email             : jiahaoyao.math@gmail.com
# Date              : 09.08.2019
# Last Modified Date: 02.03.2021
# Last Modified By  : Jiahao Yao
#
# This file is part of the VCML codebase
# Distributed under MIT license
#
# file system tools


import os
import pickle
import json
import shutil
from shutil import copy2
import ruamel.yaml as yaml
import numpy as np

def make_parent_dir(filename):
    parent_dir = os.path.dirname(filename)
    make_dir(parent_dir)


def make_dir(path):
    if not os.path.exists(path):
        os.makedirs(path, exist_ok=True)


def is_empty(path):
    return not os.path.exists(path) or (os.path.isdir(path) and len(os.listdir(path)) == 0)


def load(filename):
    if filename.endswith('.pkl'):
        with open(filename, 'rb') as f:
            loaded = pickle.load(f)
    elif filename.endswith('.json'):
        with open(filename, 'r') as f:
            loaded = json.load(f)
    elif filename.endswith('.yml') or filename.endswith('.yaml'): 
        with open(filename, 'r') as f:
            loaded = yaml.safe_load(f)
    elif filename.endswith('.txt') or filename.endswith('.sh') or filename.endswith('.bash'): 
        with open(filename, 'r') as f:
            loaded = f.read()
    else:
        raise Exception('File not recognized: %s' % filename)
    return loaded

class NumpyEncoder(json.JSONEncoder):
    # https://stackoverflow.com/questions/26646362/numpy-array-is-not-json-serializable
    def default(self, obj):
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

def dump(content, filename):
    if filename.endswith('.pkl'):
        with open(filename, 'wb') as f:
            pickle.dump(content, f, protocol=pickle.HIGHEST_PROTOCOL)
    elif filename.endswith('.json'):
        with open(filename, 'w') as f:
            json.dump(content, f, indent=4, cls=NumpyEncoder)
    elif filename.endswith('.yml') or filename.endswith('.yaml'): 
        with open(filename, 'w') as f:
            yaml.safe_dump(content, f)
    elif filename.endswith('.txt') or filename.endswith('.sh') or filename.endswith('.bash'): 
        with open(filename, 'w') as f:
            f.write(content)
    else:
        raise Exception('File not recognized: %s' % filename)


def load_knowledge(name, knowledge_type, logger=None, from_source=False):
    filename = os.path.join(
        'knowledge',
        'source' if from_source else '',
        f'{name}_{knowledge_type}.json'
    )
    if os.path.exists(filename):
        knowledge = load(filename)
    else:
        knowledge = None
    if logger is not None:
        if knowledge is not None:
            logger(f'Loading knowledge \"{knowledge_type}\" for {name} '
                   f'length = {len(knowledge)}')
        else:
            logger(f'Loading knowledge \"{knowledge_type}\", but is None')
    return knowledge


def copy_verbose(src, dst, logger=None):
    message = f'copying from {src} to {dst}'
    if logger is not None:
        logger(message)
    else:
        print(message)
    copy2(src, dst)


def copytree_verbose(src, dst, logger=None):
    shutil.copytree(src, dst, copy_function=copy_verbose)



import argparse
import datetime
import re
import urllib.request as libreq

import certifi
import dateutil.tz
from bs4 import BeautifulSoup


def parse_args():
    desc = "Get the abstract for an arxiv paper"
    parser = argparse.ArgumentParser(description=desc)
    arg_lists = []

    def add_argument_group(name):
        arg = parser.add_argument_group(name)
        arg_lists.append(arg)
        return arg

    # qunatum control
    arx_arg = add_argument_group('arxiv')
    arx_arg.add_argument('arxnum', type=str, help='arxiv number of papers')
    args = parser.parse_args()

    return args


def main():
    global args
    args = parse_args()

    # 1000 is a dummy variaable
    with libreq.urlopen('https://arxiv.org/abs/' + args.arxnum, cafile=certifi.where()) as url:
        html = url.read()
        soup = BeautifulSoup(html, 'html.parser')

    webpage = soup.get_text()

    # get the webpage
    webpage = soup.get_text()
    abstract = webpage.split('Abstract:')[1].split('Comments:')[0].strip()
    print(abstract)


if __name__ == '__main__':
    main()
